@@ -16,5 +16,6 @@ c.fill(union, [color.rgb.blue])
 c.fill(intersection, [color.rgb.red])
 c.stroke(circ1)
 c.stroke(circ2)
-c.writeEPSfile("circles")
-c.writePDFfile("circles")
+c.writeEPSfile()
+c.writePDFfile()
+c.writeSVGfile()
